﻿########################################################################### 
# 
# NAME: Clear-ADAttribute
# 
# AUTHOR: Marc Eggenberger 
# 
# COMMENT: 
# 
# VERSION HISTORY: 
# 1.0 07.10.2012 – Initial release 
# 
###########################################################################

<# 
.SYNOPSIS 
    This script clears an attribute (or several) in the AD of specific users
.DESCRIPTION 
    This script can be used to clear attributes on AD users. You can specify a search root (OU) to limit the
	affected users and specify a list of attributes that should be cleared.
.NOTES 
    File Name  : Clear-ADAttribute.ps1 
    Author     : Marc Eggenberger 
    Requires   : PowerShell V2 & Quest ActiveRoles Powershell Commands for Active Directory (Free at http://www.quest.com/powershell/activeroles-server.aspx)
.LINK 
	http://www.quest.com/powershell/activeroles-server.aspx 
.EXAMPLE 
    C:\foo> .\Clear-ADAttribute
	
	Description
	-----------
	This will connect to the default AD with the current user and clears the homeMDB and the homeMTA attribute.	 
.EXAMPLE 
    C:\foo> $cred = Get-Credential
	
	C:\foo> $conn = Connect-QADService -Credential $cred
	
	C:\foo> .\Clear-ADAttribute -QADConnection $conn
	
	Description
	-----------
	This will first prompt for a user name and password and create a QADConnecton with those credentials and then connect to the default AD
	with those credentials and clears the homeMDB and the homeMTA attribute.
.PARAMETER QADConnection 
   This is a connection you get from the connect-QADService commandlet from Quest. If not specified the script will try to
   contact the default active directory with the user the script is run. If you need to specify another username and password or
   a specific AD controller use the connect-QADService commandlet from Quest and assign it to a variable and use this variable as
   parameter here.
.PARAMETER SearchRoot 
   This is the SearchRoot the script will search users in. If not specified it will use the top level as searchroot.
.PARAMETER Attributes
   This is a string array with attributes to clear. If not specified it will clear the attributes homeMDB  and homeMTA by default.
#> 
[cmdletBinding(SupportsShouldProcess=$True)] 
param ( 
[Parameter()]  
[Quest.ActiveRoles.ArsPowerShellSnapIn.Data.ArsConnection] $QADConnection
,[Parameter()]
[string]$SearchRoot
,[Parameter()]
[String[]]$Attributes = @('homeMDB','homeMTA')
)

#Region Modules & Snapins
try {
	Write-Debug "Checking if QAD Snapin is loaded, if not load it"
	if ( (Get-PSSnapin -Name Quest.ActiveRoles.ADManagement -ErrorAction SilentlyContinue) -eq $null ) {
		Add-PSSnapin -ErrorAction Stop Quest.ActiveRoles.ADManagement
	}
} catch {
	Write-Error "Error while loading SnapIns or Modules. Error message was: $_.Exeption.Message"
	exit 1
}
#endregion

#If no connection object given try to connect with current user to default domain
if($QADConnection -eq $null) {
	try {
		Write-Debug "No connection parameter given, trying to connect to default with current user"
		$QADConnection = Connect-QADService -ErrorAction Stop
	} catch {
		Write-Error "Error while trying to connect to default AD with current user: $_.Exception.Message"
		exit 1
	}
}

#Prepare the Attribute Hashtable
$objAttributes = @{}

foreach($a in $Attributes) {
	$objAttributes.Add($a, "")
	Write-Debug "Attributes $a added to Hashtable."
}

#Get the users
if($SearchRoot -eq $null -or $SearchRoot -eq "") {
	Write-Debug "No SearchRoot found going for the top level."
	#No search root set get everything from top level
	$users = Get-QADUser -IncludedProperties $Attributes -Connection $QADConnection
	Write-Debug("Found " + $users.count + " users")
} else {
	Write-Debug "Found a Searchroot $SearchRoot."
	$users = Get-QADUser -IncludedProperties $Attributes -SearchRoot $SearchRoot -Connection $QADConnection
	Write-Debug("Found " + $users.count + " users in the $SearchRoot path")
}

#Loop over all users and save the object with the cleared attributes
foreach($user in $users) {
	Set-QADUser -Identity $user -ObjectAttributes $objAttributes -Connection $QADConnection
	Write-Debug("User " + $user.Name + " updated.")
}